/*
 * Copyright (C) 2000 by ETHZ/INF/CS
 * All rights reserved
 * 
 * @version $Id: Tsp.java,v 1.2 2010/05/29 21:02:34 cpa Exp $
 * @author Florian Schneider
 */

import java.util.*;
import java.io.*;

public class Tsp {
    public final static boolean debug = false;
    public final static int MAX_TOUR_SIZE = 32;
    public final static int MAX_NUM_TOURS = 5000;
    public final static int BIGINT = 2000000;
    public final static int END_TOUR = -1;
    public final static int ALL_DONE = -1;
    static int nWorkers = 2;
    static int TspSize = MAX_TOUR_SIZE;
    static int StartNode = 0;
    static int NodesFromEnd = 12;
    
    public static void main(String[] args) {
	int i;
	String fname = "testdata";
	boolean nop = false;
	
	try {
	    fname = args[0];
	    if (fname.equals("--nop")) nop = true;
	    else nWorkers = Integer.parseInt(args[1]);
	} catch (Exception e) {
	    System.out.println("usage: java Tsp <input file> <number of threads>\n"+
			       "    or java Tsp --nop");			       
	    System.exit(-1);
	}

	// start computation
	System.gc();
	
	Minimizer minimizer = new Minimizer();
	TourCreator tourCreator = new TourCreator(minimizer);
	
	if (!nop) {

	    tourCreator.TourStackTop = -1;
	    minimizer.MinTourLen = BIGINT;
	    
	    TspSize = read_tsp(fname, tourCreator);
	    
	    long start = new Date().getTime();
	    
	    tourCreator.initialize();
	    
	    /* create worker threads */
	    Thread[] t = new Thread[nWorkers];
	    for (i = 0; i < nWorkers; i++) {
		t[i] = new TspSolver(tourCreator, minimizer);
		
	    }
	    for (i = 0; i < nWorkers; i++) {
		t[i].start();
	    }
	    
	    /* wait for completion */
	    try {
		for (i = 0; i < nWorkers; i++) {
		    t[i].join();
		    // if (debug) System.out.println("joined thread "+i);
		}
	    } catch (InterruptedException e) {}
	    
	    long end = new Date().getTime();
	    
	    System.out.println("tsp-" + nWorkers + "\t"+ ((int) end - (int) start));
	    System.out.println("Minimum tour length: " + minimizer.MinTourLen);
	    System.out.print("Minimum tour:");
	    TspSolver.MakeTourString(TspSize, minimizer.MinTour);
	}
    }

    static int read_tsp(String fname, TourCreator tourCreator) {
	String line;
	StringTokenizer tok;
	int i, j;

	try {
	    BufferedReader in = new BufferedReader(new FileReader(fname));
	    TspSize = Integer.parseInt(in.readLine());
	    
	    for (i = 0; i < TspSize; i++) {
		line = in.readLine();
		tok = new StringTokenizer(line, " ");
		j = 0; 
		while (tok.hasMoreElements())
		    tourCreator.weights[i][j++] = Integer.parseInt((String) tok.nextElement());
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	    System.exit(-1);
	}
	return (TspSize);
    }
}
